@@ -6,12 +6,13 @@ __version__ = "Version 0.2 # 23/07/2019 # Non-Redis Bifixer # Marta Bañón"
 __version__ = "Version 0.3 # 20/08/2019 # New feature: Segmentation # Marta Bañón"
 __version__ = "Version 0.4 # 15/06/2021 # Easy install # Elsa Sarrías"
 __version__ = "Version 0.5 # 22/06/2021 # Replacements improvements and fix tokenization for Maltese # Jaume Zaragoza"
+__version__ = "Version 0.7 # 15/02/2022 # Disable punctuation normalization # Jaume Zaragoza"
 
 
 import os
 import sys
 import argparse
-import time
+import copy
 import traceback
 import logging
 
@@ -40,6 +41,7 @@ def initialization():
 
     ilines = 0
     olines = 0
+    header = "--header" in sys.argv
 
     logging.info("Processing arguments...")
     parser = argparse.ArgumentParser(prog=os.path.basename(sys.argv[0]), formatter_class=argparse.ArgumentDefaultsHelpFormatter, description=__doc__)
@@ -54,20 +56,20 @@ def initialization():
     # Target language
     parser.add_argument("trglang", type=str, help="Target language (TL) of the input")
 
-    # Mandatory parameters
-    groupM = parser.add_argument_group('Mandatory')
-
     # Options group
     groupO = parser.add_argument_group('Optional')
     # Format
-    groupO.add_argument("--scol", default=3, type=util.check_positive, help="Source sentence column (starting in 1)")
-    groupO.add_argument("--tcol", default=4, type=util.check_positive, help="Target sentence column (starting in 1)")
-    groupO.add_argument("--sdeferredcol", type=util.check_positive, help="Source deferred standoff annotation column (starting in 1)")
-    groupO.add_argument("--tdeferredcol", type=util.check_positive, help="Target deferred standoff annotation column (starting in 1)")
-       
+    groupO.add_argument("--header", action='store_true', help="Input file will have header")
+    groupO.add_argument("--scol", type=util.check_positive if not header else str, default=3 if not header else "src_text", help="Source sentence column (starting in 1). The name of the field is expected instead of the position if --header is set")
+    groupO.add_argument("--tcol", type=util.check_positive if not header else str, default=4 if not header else "trg_text", help="Target sentence column (starting in 1). The name of the field is expected instead of the position if --header is set")
+    groupO.add_argument("--sdeferredcol", type=util.check_positive if not header else str, help="Source deferred standoff annotation column (starting in 1). The name of the field is expected instead of the position if --header is set")
+    groupO.add_argument("--tdeferredcol", type=util.check_positive if not header else str, help="Target deferred standoff annotation column (starting in 1). The name of the field is expected instead of the position if --header is set")
+    groupO.add_argument("--sparagraphid", type=util.check_positive if not header else str, help="Source paragraph identification column (starting in 1). The name of the field is expected instead of the position if --header is set")
+    groupO.add_argument("--tparagraphid", type=util.check_positive if not header else str, help="Target paragraph identification column (starting in 1). The name of the field is expected instead of the position if --header is set")
 
     # Character fixing
     groupO.add_argument('--ignore_characters', default=False, action='store_true', help="Doesn't fix mojibake, orthography, or other character issues")
+    groupO.add_argument('--ignore_normalization', default=False, action='store_true', help="Doesn't normalize punctuation and spaces.")
 
     # Empty sides
     groupO.add_argument('--ignore_empty', default=False, action='store_true', help="Doesn't remove sentences with empty source or target")
@@ -122,6 +124,7 @@ def fix_sentences(args):
         chars_slang, charsRe_slang = restorative_cleaning.getCharsReplacements(args.srclang)
         chars_tlang, charsRe_tlang = restorative_cleaning.getCharsReplacements(args.trglang)
 
+    if not args.ignore_normalization:
         punctChars_slang, punctRe_slang = restorative_cleaning.getNormalizedPunctReplacements(args.srclang)
         punctChars_tlang, punctRe_tlang = restorative_cleaning.getNormalizedPunctReplacements(args.trglang)
 
@@ -137,6 +140,47 @@ def fix_sentences(args):
         source_segmenter = segmenter.NaiveSegmenter(args.srclang, args.segmenter)
         target_segmenter = segmenter.NaiveSegmenter(args.trglang, args.segmenter)
 
+    if args.header:
+        header = next(args.input).strip().split("\t")
+
+        # Transform fields to idxs
+        if args.scol not in header:
+            raise Exception(f"The provided --scol '{args.scol}' is not in the input header")
+        if args.tcol not in header:
+            raise Exception(f"The provided --tcol '{args.tcol}' is not in the input header")
+
+        args.scol = int(header.index(args.scol)) + 1
+        args.tcol = int(header.index(args.tcol)) + 1
+
+        if args.sdeferredcol:
+            if args.sdeferredcol not in header:
+                raise Exception(f"The provided --sdeferredcol '{args.sdeferredcol}' is not in the input header")
+
+            args.sdeferredcol = int(header.index(args.sdeferredcol)) + 1
+        if args.tdeferredcol:
+            if args.tdeferredcol not in header:
+                raise Exception(f"The provided --tdeferredcol '{args.tdeferredcol}' is not in the input header")
+
+            args.tdeferredcol = int(header.index(args.tdeferredcol)) + 1
+        if args.sparagraphid:
+            if args.sparagraphid not in header:
+                raise Exception(f"The provided --sparagraphid '{args.sparagraphid}' is not in the input header")
+
+            args.sparagraphid = int(header.index(args.sparagraphid)) + 1
+        if args.tparagraphid:
+            if args.tparagraphid not in header:
+                raise Exception(f"The provided --tparagraphid '{args.tparagraphid}' is not in the input header")
+
+            args.tparagraphid = int(header.index(args.tparagraphid)) + 1
+
+        # Write the output header once
+        args.output.write("\t".join(header))
+
+        if args.dedup:
+            args.output.write("\tbifixer_hash\tbifixer_score")
+
+        args.output.write("\n")
+
     for i in args.input:
         ilines += 1
         parts = i.split("\t")
@@ -144,6 +188,14 @@ def fix_sentences(args):
         try:
             source_sentence = parts[args.scol - 1]
             target_sentence = parts[args.tcol - 1]
+
+            # Check optional indexes
+            if args.sdeferredcol and args.tdeferredcol:
+                parts[args.sdeferredcol - 1]
+                parts[args.tdeferredcol - 1]
+            if args.sparagraphid and args.tparagraphid:
+                parts[args.sparagraphid - 1]
+                parts[args.tparagraphid - 1]
         except IndexError:
             logging.error(traceback.format_exc())
             logging.error("Wrong column index on line " + str(ilines))
@@ -157,12 +209,16 @@ def fix_sentences(args):
                 very_long = True
 
             if not args.ignore_characters and not very_long:
-                fixed_source = restorative_cleaning.fix(source_sentence, args.srclang, chars_slang, charsRe_slang, punctChars_slang, punctRe_slang)
-                fixed_target = restorative_cleaning.fix(target_sentence, args.trglang, chars_tlang, charsRe_tlang, punctChars_tlang, punctRe_tlang)
+                fixed_source = restorative_cleaning.fix(source_sentence, args.srclang, chars_slang, charsRe_slang)
+                fixed_target = restorative_cleaning.fix(target_sentence, args.trglang, chars_tlang, charsRe_tlang)
             else:
                 fixed_source = source_sentence.strip(" \n") 
                 fixed_target = target_sentence.strip(" \n") 
 
+            if not args.ignore_normalization and not very_long:
+                fixed_source = restorative_cleaning.normalize(fixed_source, args.srclang, punctChars_slang, punctRe_slang)
+                fixed_target = restorative_cleaning.normalize(fixed_target, args.trglang, punctChars_tlang, punctRe_tlang)
+
             if not args.ignore_orthography and not very_long:
                 corrected_source = restorative_cleaning.ortho_detok_fix(fixed_source, replacements_slang, detoks_slang)
                 corrected_target = restorative_cleaning.ortho_detok_fix(fixed_target, replacements_tlang, detoks_tlang)
@@ -180,7 +236,7 @@ def fix_sentences(args):
             sent_num = 0
             for segment in segments:
                 if not args.ignore_empty and (len(segment["source_segment"]) == 0 or len(segment["target_segment"]) == 0):
-                    continue;
+                    continue
                 if args.dedup:
                     if args.aggressive_dedup:
                         normalized_src = unidecode(segment["source_segment"].lower().translate(remove_non_alpha))
@@ -197,22 +253,33 @@ def fix_sentences(args):
                         ranking = 1
                 # if  dedupping: Add extra columnsn with hash and ranking in output file
                 # Restored parts object, with the fixed segment, overwritten for each pair of extra segments,
-                new_parts = parts
+                new_parts = copy.deepcopy(parts)
 
                 new_parts[args.scol - 1] = segment["source_segment"]
                 new_parts[args.tcol - 1] = segment["target_segment"]
-                
+
                 if len(segments) > 1:
                     sent_num += 1
-                    if args.sdeferredcol and args.tdeferredcol:
-                        if "#" in parts[args.sdeferredcol-1]:
-                            if sent_num != int(parts[args.sdeferredcol-1].split('#')[1]):
-                                continue
-                        else:
-                            new_parts[args.sdeferredcol-1] = parts[args.sdeferredcol-1].rstrip("\n")+"#"+str(sent_num)
-                            new_parts[args.tdeferredcol-1] = parts[args.tdeferredcol-1].rstrip("\n")+"#"+str(sent_num)
 
-                if args.ignore_empty or (new_parts[args.scol - 1] and new_parts[args.tcol - 1]):  # sentence sides may be empty now because it contained only spaces or similar weird thing
+                    if args.sdeferredcol and args.tdeferredcol:
+                        if "#" in parts[args.sdeferredcol - 1] or "#" in parts[args.tdeferredcol - 1]:
+                            # Reconstruction
+                            if "#" in parts[args.sdeferredcol - 1]:
+                                if sent_num != int(parts[args.sdeferredcol - 1].split('#')[1]):
+                                    continue
+                            elif "#" in parts[args.tdeferredcol - 1]:
+                                if sent_num != int(parts[args.tdeferredcol - 1].split('#')[1]):
+                                    continue
+                        else:
+                            new_parts[args.sdeferredcol - 1] = parts[args.sdeferredcol - 1].rstrip("\n") + "#" + str(sent_num)
+                            new_parts[args.tdeferredcol - 1] = parts[args.tdeferredcol - 1].rstrip("\n") + "#" + str(sent_num)
+                    if args.sparagraphid and args.tparagraphid:
+                        new_parts[args.sparagraphid - 1] = parts[args.sparagraphid - 1].rstrip("\n") + "#" + str(sent_num)
+                        new_parts[args.tparagraphid - 1] = parts[args.tparagraphid - 1].rstrip("\n") + "#" + str(sent_num)
+
+                # sentence sides may be empty now because it contained only spaces or similar weird thing
+                # for sentences containing only spaces but not normalized, strip them
+                if args.ignore_empty or (new_parts[args.scol - 1].strip() and new_parts[args.tcol - 1].strip()):
                     if (args.dedup):
                         # Remove the "/n" at the end of the last item
                         new_parts[-1] = str(new_parts[-1]).strip("\n")
